<?php

namespace Bydedi\Http;

use Bydedi\Security\Permission;
use Bydedi\Utils\Pdf\Pdf;
use Exception;
use Symfony\Component\DependencyInjection\ContainerAware;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Exception\InvalidParameterException;
use Bydedi\Http\Event\TemplateRenderEvent;
use Symfony\Component\Templating\EngineInterface;

/**
 * Class Controller
 *
 * @package Controller
 */
class Controller extends ContainerAware
{

    /**
     * @var Request
     */
    protected $request;

    /**
     * @var EngineInterface
     */
    protected $template;

    /**
     * Constructor
     *
     * @param Request $request the current request
     * @param ContainerInterface $container
     *
     */
    public function __construct(Request $request, ContainerInterface $container = null)
    {
        $this->request = $request;
        $this->setContainer($container);
        $this->template = $this->container->get('template');
    }

    /**
     * assign variable to use in the template
     *
     * @param array $data variable to assign
     *
     * @return void
     */
    public function assign(array $data)
    {
        if (!empty($data)) {
            foreach ($data as $key => $d) {
                $this->template->assign($key, $d);
            }
        }
    }

    /**
     * return instance of controller give in parameter
     *
     * @param $controllerName
     *
     * @return Controller
     *
     * @throws \Exception
     */
    public function getController($controllerName)
    {
        $controllerObject = new \ReflectionClass($controllerName);

        if ($controllerObject->isSubclassOf('Bydedi\\Http\\Controller') == false) {
            throw new Exception('the class ' . $controllerName . ' must extend Controller class');
        }

        return new $controllerName($this->request, $this->container);
    }

    /**
     * Permet de généré une url
     *
     * @param string $name          the name of the url
     * @param array  $parameters    array of parameters for generate the url
     * @param string $referenceType The type of reference to be generated
     *
     * @return string
     */
    public function generateUrl($name, $parameters = array(), $referenceType = null)
    {
        try {
            return $this->container->get('router')->generate($name, $parameters, $referenceType);
        } catch (InvalidParameterException $e) {
            return false;
        }
    }

    /**
     * Check if the current user has permission give in paramter
     *
     * @param $permissionCode
     *
     * @return mixed
     *
     * @throws Exception
     */
    public function hasPermission($permissionCode)
    {
        $locker = $this->container->get('security.locker');
        $permission = $locker->findPermissionByCode($permissionCode);

        if (!$permission instanceof Permission) {
            throw new Exception('permission code "' . $permissionCode . '" doesn\'t exist');
        }

        $roles = $this->getSession()->get('roles');

        return $locker->can($roles, $permission);
    }

    /**
     * Fonction pour afficher un template
     *
     * @param string $tpl  name of the tpl to display
     * @param array  $data array of data to assign for use in the template
     *
     * @return mixed
     *
     * @throws \Exception
     * @throws \SmartyException
     */
    public function render($tpl, $data = array())
    {
        if (!empty($data)) {
            $this->assign($data);
        }

        $this->container->get('event.dispatcher')->dispatch(
            HttpEvents::VIEW,
            new TemplateRenderEvent($this->template)
        );

        // on enregistre passe les infos de session au template
        $this->template->assign('session', $this->request->getSession()->all());

        return $this->response($this->template->render($tpl));
    }

    /**
     * permet de rendre un bout de template, sans le layout
     *
     * @param string $tpl  name of the tpl to display
     * @param array  $data array of data to assign for use in the template
     *
     * @return Response
     *
     * @throws \Exception
     * @throws \SmartyException
     */
    public function fetch($tpl, $data = array())
    {
        if (!empty($data)) {
            $this->assign($data);
        }

        return $this->response($this->template->render($tpl));
    }

    /**
     * Function pour générer un PDF
     *
     * @param Pdf $pdf
     * @param null $filename
     *
     * @return Response
     */
    public function renderPdf(Pdf $pdf, $filename = null)
    {

        if (is_null($filename)) {
            $filename = "file.pdf";
        }

        $headers = [
            'Content-Type' => 'application/pdf',
            'Content-Disposition' => 'attachment; filename="'.$filename.'"',
        ];

        return $this->response($pdf->output(), 200, $headers);
    }

    /**
     * Génère une réponse à renvoyer
     *
     * @param string $content contenu de la réponse
     * @param int    $status  code HTTP de la réponse
     * @param array  $headers headers complémentaire
     *
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function response($content = '', $status = 200, $headers = array())
    {
        $objResponse = (in_array($this->request->getMimeType('json'), $this->request->getAcceptableContentTypes())) ?
            'Symfony\Component\HttpFoundation\JsonResponse' : 'Symfony\Component\HttpFoundation\Response';
        return new $objResponse($content, $status, $headers);
    }

    /**
     * Permet de récupérer la session de la requete courante
     *
     * @return \Symfony\Component\HttpFoundation\Session\SessionInterface
     */
    public function getSession()
    {
        return $this->request->getSession();
    }

    /**
     * Permet de récuperer le flashBag lié a la requete courante
     *
     * @return \Symfony\Component\HttpFoundation\Session\Flash\FlashBagInterface
     */
    public function getFlashBag()
    {
        return $this->getSession()->getFlashBag();
    }

    /**
     * Permet de faire une redirection vers une url
     *
     * @param string $url     l'url vers laquelle on veut rediriger l'internaute
     * @param int    $status  le code HTTP
     * @param array  $headers des headers complémentaire
     *
     * @return RedirectResponse
     */
    public function redirect($url, $status = 302, $headers = array())
    {
        return new RedirectResponse($url, $status, $headers);
    }
}
